#include "coordinate_system.h"

#define X fields[0][0]->value()
#define Y fields[0][1]->value()

CoordinateSystem::CoordinateSystem(QWidget *parent) : QWidget(parent) {
    QGridLayout *layout = new QGridLayout();
    
    QLabel* xy = new QLabel("XY:", this);
    QLabel* iters = new QLabel("Iters:", this);
    QLabel* stepLabel = new QLabel("Step:", this);
    QLabel* angleLabel = new QLabel("Angle:", this);
    QLabel* colorLabel = new QLabel("Color:", this);
    QLabel* cxLabel = new QLabel("cx:", this);
    QLabel* cyLabel = new QLabel("cy:", this);
    layout->addWidget(xy, 0, 0);
    layout->addWidget(iters, 1, 0);
    layout->addWidget(stepLabel, 2, 0);
    layout->addWidget(angleLabel, 3, 0);
    layout->addWidget(colorLabel, 4, 0);
    for (char i = 0; i < 3; ++i) {
        for (char j = 0; j < 2; ++j) {
            fields[i][j] = new QSpinBox(this);
            fields[i][j]->setFixedWidth(60);
        }
    }
    for (int j = 0; j < 2; ++j) {
        fields[0][j]->setSingleStep(10);
        layout->addWidget(fields[0][j], 0, j + 1);
        connect(fields[0][j], QOverload<int>::of(&QSpinBox::valueChanged), this, &CoordinateSystem::up);
    }
    fields[1][0]->setRange(1, 16);
    fields[1][0]->setValue(9);
    layout->addWidget(fields[1][0], 1, 1);
    
    fields[1][1]->setRange(5, 50);
    fields[1][1]->setValue(10);
    fields[1][1]->setSingleStep(5);
    layout->addWidget(fields[1][1], 2, 1);
    
    fields[2][0]->setRange(0, 3);
    layout->addWidget(fields[2][0], 3, 1);
    
    fields[2][1]->setRange(0, 359);
    fields[2][1]->setSingleStep(10);
    layout->addWidget(fields[2][1], 4, 1);

    fields[1][0]->setRange(1, 999);
    fields[1][0]->setValue(9);
    button = new QPushButton("Dragon", this);    
    QPushButton *clearb = new QPushButton("Clear", this);
    clearb->setFixedWidth(60);
    layout->addWidget(clearb,4,2);
    layout->addWidget(button,5,1,1,2);
    QWidget *container = new QWidget(this);
    container->setLayout(layout);
    connect(button, &QPushButton::clicked, this, &CoordinateSystem::onButtonClick);
    connect(clearb, &QPushButton::clicked, this, &CoordinateSystem::clear);
    fields[2][1]->setSuffix("°");
    fields[2][0]->setSuffix("π");
    QPushButton *juluar = new QPushButton("Jul", this);
    layout->addWidget(juluar, 2, 3, 1, 2);
    connect(juluar, &QPushButton::clicked, this, &CoordinateSystem::juluar);
    cx.setRange(-2, 2);
    cx.setSingleStep(0.1);
    cx.setValue(-0.7);
    cy.setRange(-2, 2);
    cy.setSingleStep(0.1);
    cy.setValue(0.27015);
    layout->addWidget(&cx, 0, 4);
    layout->addWidget(&cy, 1, 4);
    layout->addWidget(cxLabel, 0, 3);
    layout->addWidget(cyLabel, 1, 3);
}
void CoordinateSystem::up() {update();}
void CoordinateSystem::clear() {lines.clear();imgs.clear();update();}
void CoordinateSystem::paintEvent(QPaintEvent*) {
  short newMin1 = -centerX;
  if (fields[0][0]->minimum() != newMin1){
              fields[0][0]->blockSignals(true);
              fields[0][0]->setRange(newMin1, -newMin1);
              fields[0][0]->blockSignals(false);
  }
  short newMin = centerY;
  if (fields[0][1]->maximum() != newMin){
              fields[0][1]->blockSignals(true);
              fields[0][1]->setRange(-newMin,newMin);
              fields[0][1]->blockSignals(false);}
    painter = new QPainter(this);
    for (unsigned char i=0;i<imgs.size();i++){
        painter->drawImage(0,0,imgs[i]);
    }
    painter->drawLine(0, centerY, width(), centerY);
    painter->drawLine(centerX, 0, centerX, height());
    painter->drawLine(centerX - 5, 11, centerX, 0);
    painter->drawLine(centerX + 5, 11, centerX, 0);
    painter->drawLine(width()-11, centerY - 5, width(), centerY);
    painter->drawLine(width()-11, centerY + 5, width(), centerY);
    for (short i = centerX; i < width(); i += step)  
        painter->drawLine(i, centerY - 5, i, centerY + 5);
    for (short i = centerX; i > 0; i -= step)        
        painter->drawLine(i, centerY - 5, i, centerY + 5);
    for (short i = centerY; i < height(); i += step)
        painter->drawLine(centerX - 5, i, centerX + 5, i);
    for (short i = centerY; i > 0; i -= step) 
        painter->drawLine(centerX - 5, i, centerX + 5, i);
    for (char a = 0; a < lines.size() / 6; a++) {
        QVector<QPoint> points;
        points.append(QPoint(0, 0));
        points.append(QPoint(lines[a*6+3], 0));
    
        for (int i = 0; i < lines[a*6+2]; ++i) {
            QPoint pivot = points.last();
            for (int j = points.size() - 2; j >= 0; --j) {
                QPoint vec = points[j] - pivot;
                points.append(pivot + QPoint(-vec.y(), vec.x()));
            }
        }
        painter->setPen(QPen(QColor::fromHsv(lines[a*6+5], 255, 255)));
        for (int i = 0; i < points.size() - 1; ++i) {
            QPoint p1 = QPoint(points[i].x() * cos(lines[a*6+4]*M_PI/2) - points[i].y() * sin(lines[a*6+4]*M_PI/2), points[i].x() * sin(lines[a*6+4]*M_PI/2) + points[i].y() * cos(lines[a*6+4]*M_PI/2)) + QPoint(lines[a*6] + centerX, centerY - lines[a*6+1]);
            QPoint p2 = QPoint(points[i+1].x() * cos(lines[a*6+4]*M_PI/2) - points[i+1].y() * sin(lines[a*6+4]*M_PI/2), points[i+1].x() * sin(lines[a*6+4]*M_PI/2) + points[i+1].y() * cos(lines[a*6+4]*M_PI/2)) + QPoint(lines[a*6] + centerX, centerY - lines[a*6+1]);
            painter->drawLine(p1, p2);
        }
    }

    painter->setPen(Qt::white);painter->setBrush(Qt::NoBrush);
    painter->drawText(centerX - step + 5, centerY - 5, QString::number(-step));
    painter->drawText(centerX + 5, centerY - 5 + step, QString::number(-step));
    painter->drawText(centerX + 5, centerY - 5 - step, ' ' + QString::number(step));
    painter->drawText(centerX + 5, 15, "Y");
    painter->drawText(width()-15, centerY - 5, "X");
    painter->drawText(centerX + 5, centerY - 5, "0");
    painter->drawText(centerX + step + 5, centerY - 5, QString::number(step));
    painter->drawEllipse(centerX + X-5, centerY - Y-5, 11, 11);
    painter->end();
}

void CoordinateSystem::onButtonClick() {
  lines.push_back(X);lines.push_back(Y);
  lines.push_back(fields[1][0]->value());
  lines.push_back(fields[1][1]->value());
  lines.push_back((fields[2][0]->value()));
  lines.push_back(fields[2][1]->value());
  update();
}
void CoordinateSystem::juluar() {
  imgs.clear();
  QImage img(width(), height(), QImage::Format_RGB16);
  
      for (unsigned short x = 0; x < width(); ++x) {
          for (unsigned short y = 0; y < height(); ++y) {
              float zx = 1.5 * (x - centerX) / float(/* zoom */ centerX)/* + moveX*/;
              float zy = (y - centerY) / float(/* zoom */ centerY)/* + moveY*/;
              int i = 0;
              float zx2 = zx * zx;
              float zy2 = zy * zy;
              while (zx2 + zy2 < 4.0f && i < fields[1][0]->value()) {
                  float tmp = zx2 - zy2 + cx.value();
                  zy = 2.0f * zx * zy + cy.value();
                  zx = tmp;
              
                  zx2 = zx * zx;
                  zy2 = zy * zy;
                  ++i;
              }
              QColor color;
              if (i == fields[1][0]->value())
                  color = Qt::black;
              else
                  color = QColor::fromHsv(short(i * sqrt(sqrt(fields[1][0]->value())*log2(fields[1][0]->value()))) % 360, 255, 255);
              img.setPixel(x, y, color.rgb());
          }
      }
  imgs.push_back(img);
    update();
}
